from unittest.mock import call, patch

from django.test import override_settings

from arkindex.ponos.models import Farm
from arkindex.process.models import ProcessMode, WorkerVersion
from arkindex.project.tests import FixtureTestCase


class TestModels(FixtureTestCase):

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        cls.farm = Farm.objects.first()
        cls.process = cls.corpus.processes.create(
            creator=cls.user,
            mode=ProcessMode.Workers,
            corpus=cls.corpus,
            farm=cls.farm,
        )
        cls.worker_version1 = WorkerVersion.objects.get(worker__slug="reco")
        cls.worker_version2 = WorkerVersion.objects.get(worker__slug="dla")
        cls.run1 = cls.process.worker_runs.create(version=cls.worker_version1, parents=[])
        cls.run2 = cls.process.worker_runs.create(version=cls.worker_version2, parents=[cls.run1.id])

    @override_settings(PONOS_RQ_EXECUTION=True)
    @patch("arkindex.ponos.tasks.run_task_rq.delay")
    def test_run_process_schedules_tasks(self, run_task_mock):
        run_task_mock.side_effect = ["init_job", "job1", "job2"]
        # on_commit will not be called automatically as savepoints are used for tests transactions
        with (
            self.captureOnCommitCallbacks(execute=True),
            patch("arkindex.process.tasks.initialize_activity.delay"),
        ):
            self.process.run()
        init, t1, t2 = self.process.tasks.order_by("depth")
        self.assertEqual(run_task_mock.call_count, 3)
        call1, call2, call3 = run_task_mock.call_args_list
        self.assertEqual(call1, call(init))
        self.assertTupleEqual(call2.args, (t1,))
        self.assertListEqual(list(call2.kwargs.keys()), ["depends_on"])
        task1_depends = call2.kwargs["depends_on"]
        self.assertEqual(vars(task1_depends), {
            "dependencies": ["init_job"],
            "allow_failure": True,
            "enqueue_at_front": False,
        })
        self.assertTupleEqual(call3.args, (t2,))
        self.assertListEqual(list(call3.kwargs.keys()), ["depends_on"])
        task2_depends = call3.kwargs["depends_on"]
        self.assertEqual(vars(task2_depends), {
            "dependencies": ["job1"],
            "allow_failure": True,
            "enqueue_at_front": False,
        })
