from django.urls import path
from django.views.generic.base import RedirectView

from arkindex.documents.api.elements import (
    AllowedMetaDataEdit,
    CorpusAllowedMetaData,
    CorpusElements,
    CorpusList,
    CorpusRetrieve,
    CorpusSelectionDestroy,
    ElementBulkCreate,
    ElementChildren,
    ElementChildrenCreate,
    ElementMetadata,
    ElementMetadataBulk,
    ElementMove,
    ElementNeighbors,
    ElementParent,
    ElementParents,
    ElementRetrieve,
    ElementsCreate,
    ElementTranscriptions,
    ElementTypeCreate,
    ElementTypeUpdate,
    ManageSelection,
    MetadataEdit,
    SelectionMove,
    SelectionParentCreate,
    WorkerResultsDestroy,
)
from arkindex.documents.api.entities import (
    CorpusEntities,
    CorpusEntityTypes,
    EntityCreate,
    EntityDetails,
    EntityElements,
    EntityTypeCreate,
    EntityTypeUpdate,
    TranscriptionEntities,
    TranscriptionEntitiesBulk,
    TranscriptionEntityCreate,
)
from arkindex.documents.api.export import CorpusExportAPIView, ManageExport
from arkindex.documents.api.iiif import ElementAnnotationList, FolderManifest
from arkindex.documents.api.ml import (
    ClassificationBulk,
    ClassificationCreate,
    ClassificationReject,
    ClassificationValidate,
    CorpusMLClassList,
    ElementTranscriptionsBulk,
    ManageClassificationsSelection,
    MLClassRetrieve,
    TranscriptionBulk,
    TranscriptionCreate,
    TranscriptionEdit,
)
from arkindex.documents.api.search import CorpusSearch, SearchIndexBuild
from arkindex.images.api import IIIFInformationCreate, IIIFURLCreate, ImageCreate, ImageElements, ImageRetrieve
from arkindex.ponos.api import TaskArtifactDownload, TaskArtifacts, TaskDetails, TaskRestart
from arkindex.process.api import (
    ApplyProcessTemplate,
    BucketList,
    ClearProcess,
    CorpusProcess,
    CorpusWorkerRunList,
    CorpusWorkersActivity,
    CorpusWorkerVersionList,
    CreateProcessTemplate,
    DataFileCreate,
    DataFileList,
    DataFileRetrieve,
    FeatureWorkerVersionRetrieve,
    FilesProcess,
    ListProcessElements,
    ProcessDatasetSetManage,
    ProcessDatasetSets,
    ProcessDetails,
    ProcessFailures,
    ProcessList,
    ProcessRetry,
    ProcessWorkersActivity,
    S3ImportCreate,
    SelectProcessFailures,
    StartProcess,
    UpdateWorkerActivity,
    UserWorkerRunCreate,
    UserWorkerRunList,
    WorkerActivityList,
    WorkerConfigurationList,
    WorkerConfigurationRetrieve,
    WorkerList,
    WorkerRetrieve,
    WorkerRunDetails,
    WorkerRunList,
    WorkerTypesList,
    WorkerVersionList,
    WorkerVersionRetrieve,
)
from arkindex.project.openapi import OpenApiSchemaView
from arkindex.training.api import (
    CorpusDataset,
    CreateDatasetElementsSelection,
    DatasetClone,
    DatasetElementDestroy,
    DatasetElements,
    DatasetPopulate,
    DatasetSetCreate,
    DatasetSets,
    DatasetUpdate,
    ElementDatasetSets,
    ModelCompatibleWorkerManage,
    ModelRetrieve,
    ModelsList,
    ModelVersionDownload,
    ModelVersionsList,
    ModelVersionsRetrieve,
)
from arkindex.users.api import (
    JobList,
    JobRetrieve,
    PasswordReset,
    PasswordResetConfirm,
    SendVerificationEmail,
    UserCreate,
    UserEmailLogin,
    UserEmailVerification,
    UserRetrieve,
)

api = [

    # Elements
    path("elements/<uuid:pk>/neighbors/", ElementNeighbors.as_view(), name="elements-neighbors"),
    path("elements/<uuid:pk>/parents/", ElementParents.as_view(), name="elements-parents"),
    path("elements/<uuid:pk>/children/", ElementChildren.as_view(), name="elements-children"),
    path("elements/selection/", ManageSelection.as_view(), name="elements-selection"),
    path("elements/selection/move/", SelectionMove.as_view(), name="move-selection"),
    path("elements/selection/parent/", SelectionParentCreate.as_view(), name="create-parent-selection"),
    path("elements/create/", ElementsCreate.as_view(), name="elements-create"),
    path("elements/type/", ElementTypeCreate.as_view(), name="element-type-create"),
    path("elements/type/<uuid:pk>/", ElementTypeUpdate.as_view(), name="element-type"),
    path("element/<uuid:pk>/", ElementRetrieve.as_view(), name="element-retrieve"),
    path("element/<uuid:pk>/children/bulk/", ElementBulkCreate.as_view(), name="elements-bulk-create"),
    path("element/<uuid:pk>/metadata/", ElementMetadata.as_view(), name="element-metadata"),
    path("element/<uuid:pk>/metadata/bulk/", ElementMetadataBulk.as_view(), name="element-metadata-bulk"),
    path("element/<uuid:pk>/transcription/", TranscriptionCreate.as_view(), name="transcription-create"),
    path("element/<uuid:pk>/transcriptions/", ElementTranscriptions.as_view(), name="element-transcriptions"),
    path(
        "element/<uuid:pk>/transcriptions/bulk/",
        ElementTranscriptionsBulk.as_view(),
        name="element-transcriptions-bulk"
    ),
    path("element/<uuid:child>/parent/<uuid:parent>/", ElementParent.as_view(), name="element-parent"),
    path("element/parent/<uuid:pk>/", ElementChildrenCreate.as_view(), name="element-children-create"),
    path("element/move/", ElementMove.as_view(), name="move-element"),

    # Corpora
    path("corpus/", CorpusList.as_view(), name="corpus"),
    path("corpus/<uuid:pk>/", CorpusRetrieve.as_view(), name="corpus-retrieve"),
    # `corpus` and not `pk` so that OpenAPI uses a `corpus` parameter
    path("corpus/<uuid:corpus>/elements/", CorpusElements.as_view(), name="corpus-elements"),
    path("corpus/<uuid:pk>/classes/", CorpusMLClassList.as_view(), name="corpus-classes"),
    path("corpus/<uuid:corpus>/classes/<uuid:mlclass>/", MLClassRetrieve.as_view(), name="ml-class-retrieve"),
    path("corpus/<uuid:pk>/entity-types/", CorpusEntityTypes.as_view(), name="corpus-entity-types"),
    path("corpus/<uuid:pk>/entities/", CorpusEntities.as_view(), name="corpus-entities"),
    path("corpus/<uuid:pk>/allowed-metadata/", CorpusAllowedMetaData.as_view(), name="corpus-allowed-metadata"),
    path("corpus/<uuid:corpus>/allowed-metadata/<uuid:pk>/", AllowedMetaDataEdit.as_view(), name="allowed-metadata-edit"),
    path("corpus/<uuid:pk>/versions/", CorpusWorkerVersionList.as_view(), name="corpus-versions"),
    path("corpus/<uuid:pk>/worker-runs/", CorpusWorkerRunList.as_view(), name="corpus-runs"),
    path("corpus/<uuid:pk>/selection/", CorpusSelectionDestroy.as_view(), name="corpus-delete-selection"),
    path("corpus/<uuid:pk>/search/", CorpusSearch.as_view(), name="corpus-search"),
    path("corpus/<uuid:pk>/export/", CorpusExportAPIView.as_view(), name="corpus-export"),
    path("corpus/<uuid:corpus>/worker-results/", WorkerResultsDestroy.as_view(), name="worker-delete-results"),
    path("corpus/<uuid:corpus>/activity-stats/", CorpusWorkersActivity.as_view(), name="corpus-activity-stats"),
    path("corpus/<uuid:corpus>/activity/", WorkerActivityList.as_view(), name="corpus-activity"),
    path("corpus/<uuid:pk>/index/", SearchIndexBuild.as_view(), name="build-search-index"),
    path("export/<uuid:pk>/", ManageExport.as_view(), name="manage-export"),

    # Datasets
    path("corpus/<uuid:pk>/datasets/", CorpusDataset.as_view(), name="corpus-datasets"),
    path("corpus/<uuid:pk>/datasets/selection/", CreateDatasetElementsSelection.as_view(), name="dataset-elements-selection"),
    path("element/<uuid:pk>/sets/", ElementDatasetSets.as_view(), name="element-sets"),
    path("datasets/<uuid:pk>/", DatasetUpdate.as_view(), name="dataset-update"),
    path("datasets/<uuid:pk>/clone/", DatasetClone.as_view(), name="dataset-clone"),
    path("datasets/<uuid:pk>/elements/", DatasetElements.as_view(), name="dataset-elements"),
    path("datasets/<uuid:dataset>/elements/<uuid:element>/", DatasetElementDestroy.as_view(), name="dataset-element"),
    path("datasets/<uuid:pk>/sets/", DatasetSetCreate.as_view(), name="dataset-sets"),
    path("datasets/<uuid:dataset>/sets/<uuid:set>/", DatasetSets.as_view(), name="dataset-set"),
    path("datasets/<uuid:pk>/populate/", DatasetPopulate.as_view(), name="dataset-populate"),

    # Moderation
    path("classifications/", ClassificationCreate.as_view(), name="classification-create"),
    path("classifications/<uuid:pk>/validate/", ClassificationValidate.as_view(), name="classification-validate"),
    path("classifications/<uuid:pk>/reject/", ClassificationReject.as_view(), name="classification-reject"),
    path("classifications/selection/", ManageClassificationsSelection.as_view(), name="classification-selection"),

    # Manifests
    path("iiif/<uuid:pk>/manifest/", FolderManifest.as_view(), name="folder-manifest"),
    path("iiif/<uuid:pk>/list/transcriptions/", ElementAnnotationList.as_view(), name="element-annotation-list"),

    # Placeholder URLs for IIIF IDs
    path(
        "iiif/<uuid:pk>/sequence/",
        RedirectView.as_view(pattern_name="api:folder-manifest", permanent=False),
        name="iiif-sequence",
    ),
    path(
        "iiif/<uuid:pk>/canvas/",
        RedirectView.as_view(pattern_name="api:element-annotation-list", permanent=False),
        name="iiif-canvas",
    ),
    path(
        "iiif/<uuid:page_pk>/annotation/<uuid:transcription_pk>/",
        RedirectView.as_view(pattern_name="api:element-annotation-list", permanent=False),
        name="transcription-annotation",
    ),

    # Ingest transcriptions
    path("transcription/<uuid:pk>/", TranscriptionEdit.as_view(), name="transcription-edit"),
    path("transcription/bulk/", TranscriptionBulk.as_view(), name="transcription-bulk"),
    path("classification/bulk/", ClassificationBulk.as_view(), name="classification-bulk"),

    # Ingest and retrieve entities
    path("entity/types/", EntityTypeCreate.as_view(), name="entity-type-create"),
    path("entity/types/<uuid:pk>/", EntityTypeUpdate.as_view(), name="entity-type-details"),
    path("entity/", EntityCreate.as_view(), name="entity-create"),
    path("entity/<uuid:pk>/", EntityDetails.as_view(), name="entity-details"),
    path("entity/<uuid:pk>/elements/", EntityElements.as_view(), name="entity-elements"),
    path("transcription/<uuid:pk>/entity/", TranscriptionEntityCreate.as_view(), name="transcription-entity-create"),
    path("transcription/<uuid:pk>/entities/", TranscriptionEntities.as_view(), name="transcription-entities"),
    path("transcription/<uuid:pk>/entities/bulk/", TranscriptionEntitiesBulk.as_view(), name="transcription-entities-bulk"),

    # Ingest metadata
    path("metadata/<uuid:pk>/", MetadataEdit.as_view(), name="metadata-edit"),

    # Workers
    path("workers/", WorkerList.as_view(), name="workers-list"),
    path("workers/types/", WorkerTypesList.as_view(), name="worker-type-list"),
    path("workers/<uuid:pk>/", WorkerRetrieve.as_view(), name="worker-retrieve"),
    path("workers/<uuid:pk>/configurations/", WorkerConfigurationList.as_view(), name="worker-configurations"),
    path("workers/<uuid:pk>/versions/", WorkerVersionList.as_view(), name="worker-versions"),
    path("workers/versions/<uuid:pk>/", WorkerVersionRetrieve.as_view(), name="version-retrieve"),
    path("workers/versions/<uuid:pk>/activity/", UpdateWorkerActivity.as_view(), name="update-worker-activity"),
    path("workers/versions/feature/<feature>/", FeatureWorkerVersionRetrieve.as_view(), name="feature-worker-version"),
    path("workers/configurations/<uuid:pk>/", WorkerConfigurationRetrieve.as_view(), name="configuration-retrieve"),

    # Import workflows
    path("process/", ProcessList.as_view(), name="process-list"),
    path("process/fromfiles/", FilesProcess.as_view(), name="files-process"),
    path("process/<uuid:pk>/", ProcessDetails.as_view(), name="process-details"),
    path("process/<uuid:pk>/retry/", ProcessRetry.as_view(), name="process-retry"),
    path("process/<uuid:pk>/start/", StartProcess.as_view(), name="process-start"),
    path("process/<uuid:pk>/process-failures/", ProcessFailures.as_view(), name="process-failures"),
    path("process/files/<uuid:pk>/", DataFileList.as_view(), name="file-list"),
    path("process/files/create/", DataFileCreate.as_view(), name="file-create"),
    path("process/file/<uuid:pk>/", DataFileRetrieve.as_view(), name="file-retrieve"),
    path("process/corpus/", CorpusProcess.as_view(), name="corpus-process"),
    path("process/<uuid:pk>/workers/", WorkerRunList.as_view(), name="worker-run-list"),
    path("process/workers/<uuid:pk>/", WorkerRunDetails.as_view(), name="worker-run-details"),
    path("process/local/workers/", UserWorkerRunList.as_view(), name="user-worker-run-list"),
    path("process/local/workers/create/", UserWorkerRunCreate.as_view(), name="user-worker-run-create"),
    path("process/<uuid:pk>/elements/", ListProcessElements.as_view(), name="process-elements-list"),
    path("process/<uuid:pk>/activity-stats/", ProcessWorkersActivity.as_view(), name="process-activity-stats"),
    path("process/<uuid:pk>/template/", CreateProcessTemplate.as_view(), name="create-process-template"),
    path("process/<uuid:pk>/apply/", ApplyProcessTemplate.as_view(), name="apply-process-template"),
    path("process/<uuid:pk>/clear/", ClearProcess.as_view(), name="clear-process"),
    path("process/<uuid:pk>/select-failures/", SelectProcessFailures.as_view(), name="process-select-failures"),
    path("process/<uuid:pk>/sets/", ProcessDatasetSets.as_view(), name="process-sets"),
    path("process/<uuid:process>/set/<uuid:set>/", ProcessDatasetSetManage.as_view(), name="process-set"),

    # ML models training
    path("modelversion/<uuid:pk>/", ModelVersionsRetrieve.as_view(), name="model-version-retrieve"),
    path("models/", ModelsList.as_view(), name="models"),
    path("model/<uuid:pk>/", ModelRetrieve.as_view(), name="model-retrieve"),
    path("model/<uuid:model>/compatible-worker/<uuid:worker>/", ModelCompatibleWorkerManage.as_view(), name="model-compatible-worker-manage"),
    path("model/<uuid:pk>/versions/", ModelVersionsList.as_view(), name="model-versions"),
    path("modelversion/<uuid:pk>/download/", ModelVersionDownload.as_view(), name="model-version-download"),

    # S3 ingest
    path("ingest/buckets/", BucketList.as_view(), name="bucket-list"),
    path("ingest/create/", S3ImportCreate.as_view(), name="s3-import-create"),

    # Image management
    path("image/", ImageCreate.as_view(), name="image-create"),
    path("image/iiif/url/", IIIFURLCreate.as_view(), name="iiif-url-create"),
    path("image/iiif/info/", IIIFInformationCreate.as_view(), name="iiif-info-create"),
    path("image/<uuid:pk>/", ImageRetrieve.as_view(), name="image-retrieve"),
    path("image/<uuid:pk>/elements/", ImageElements.as_view(), name="image-elements"),

    # Authentication
    path("user/", UserRetrieve.as_view(), name="user-retrieve"),
    path("user/new/", UserCreate.as_view(), name="user-new"),
    path("user/login/", UserEmailLogin.as_view(), name="user-login"),
    path("user/verify-email/", UserEmailVerification.as_view(), name="verify-email"),
    path("user/verify-email/send/", SendVerificationEmail.as_view(), name="send-verification-email"),
    path("user/password-reset/", PasswordReset.as_view(), name="password-reset"),
    path("user/password-reset/confirm/", PasswordResetConfirm.as_view(), name="password-reset-confirm"),

    # Asynchronous jobs
    path("jobs/", JobList.as_view(), name="jobs-list"),
    path("jobs/<path:pk>/", JobRetrieve.as_view(), name="jobs-retrieve"),

    # OpenAPI Schema
    path("openapi/", OpenApiSchemaView.as_view(), name="openapi-schema"),

    # Ponos
    path("task/<uuid:pk>/", TaskDetails.as_view(), name="task-details"),
    path(
        "task/<uuid:pk>/artifacts/", TaskArtifacts.as_view(), name="task-artifacts"
    ),
    path(
        "task/<uuid:pk>/artifact/<path:path>",
        TaskArtifactDownload.as_view(),
        name="task-artifact-download",
    ),
    path("task/<uuid:pk>/restart/", TaskRestart.as_view(), name="task-restart"),
]
